#!/usr/bin/env python

"""
Submit one or more reports from local files (txt, pdf)
"""
from __future__ import print_function

import argparse
import os

from trustar import TruStar


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=(
                                         'Submit one or more reports from local files (txt, pdf, docx, etc) '
                                         'in a directory\n\n'
                                         'Example:\n'
                                         'python bulk_upload.py ./sample_reports'))
    parser.add_argument('dir', help='Path containing report files')

    args = parser.parse_args()
    source_report_dir = args.dir

    ts = TruStar(config_role="trustar")
    token = ts.get_token()

    # process all files in directory
    print("Processing and submitting each source file in %s as a TruSTAR Incident Report" % source_report_dir)

    for i in range(1, 100):
        for (dirpath, dirnames, filenames) in os.walk(source_report_dir):
            for source_file in filenames:
                print("Processing source file %s " % source_file)
                try:
                    path = os.path.join(source_report_dir, source_file)
                    report_body_txt = ts.process_file(path)

                    # response_json = ts.submit_report(token, report_body_txt, "COMMUNITY: " + file)
                    response_json = ts.submit_report(token, report_body_txt, "ENCLAVE: " + source_file, enclave=True)
                    report_id = response_json['reportId']

                    print("SUCCESSFULLY SUBMITTED REPORT, TRUSTAR REPORT as Incident Report ID %s" % report_id)


                except Exception as e:
                    print("Problem with file %s, exception: %s " % (source_file, e))
                    continue

                    # time.sleep(2)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from __future__ import print_function

"""
Process a FireEye alerts API export csv and convert each row to an incident report and submitting to TruSTAR:

"""
import json
import sys
import time

import pandas as pd

from trustar import TruStar

# Set to false to submit to community
do_enclave_submissions = True


def filter_false_positive(df, process_time):
    """
    method that takes in FireEye (FE) alerts and filters FE-tests and False Positives
    :param process_time:
    :param df:
    :return:
    """
    result = []
    track = []
    count = 0
    for o in df['alerts']:
        count += 1
        if 'closedState' in o:
            if o['closedState'] != 'False Positive':
                if 'distinguishers' in o:
                    try:
                        if 'virus' in o['distinguishers']:
                            if o['distinguishers']['virus'] != 'fetestevent':
                                result.append(o)
                            else:
                                track.append(o)  # Track fetestevents that are skipped
                        else:
                            result.append(o)
                    except TypeError:
                        result.append(o)
                else:
                    result.append(o)
            else:
                track.append(o)  # Track false positives that are skipped
        elif 'distinguishers' in o:
            try:
                if 'virus' in o['distinguishers']:
                    if o['distinguishers']['virus'] != 'fetestevent':
                        result.append(o)
                    else:
                        track.append(o)  # Track fetestevents that are skipped
                else:
                    result.append(o)
            except TypeError:
                result.append(o)

    trackfile = open('tracking_fetest_' + process_time + '.txt', 'w')
    numskip = 1
    for item in track:
        trackfile.write("\n\n**** {:d}: Display ID {} ****\n\n{}".format(numskip, item['displayId'], item))
        numskip += 1
    return result


def filter_win_methodology(df, process_time):
    """
    A function that filters out the windows methodology alert data obtained from FireEye
    :param df: a DataFrame object
    :param process_time:
    :return:
    """
    result = []
    track = []
    for o in df:
        if 'WINDOWS METHODOLOGY' in o['message']:
            track.append(o)
        else:
            result.append(o)
    trackfile = open('tracking_winMethodology_' + process_time + '.txt', 'w')
    numskip = 1
    for item in track:
        trackfile.write("\n\n**** {:d}: Display ID {} ****\n\n{}".format(numskip, item['displayId'], item))
        numskip += 1
    return result


def filter_bash_shellshock(df, process_time):
    """
    A function that filters out the BASH SHELLSHOCK alert data obtained from FireEye
    :param df: a DataFrame object
    :param process_time:
    :return:
    """
    result = []
    track = []
    for o in df:
        if 'BASH [Shellshock HTTP]' in o['message']:
            track.append(o)
        else:
            result.append(o)
    trackfile = open('tracking_bashShellShock_' + process_time + '.txt', 'w')
    numskip = 1
    for item in track:
        trackfile.write("\n\n**** {:d}: Display ID {} ****\n\n{}".format(numskip, item['displayId'], item))
        numskip += 1
    return result


def filter_webapp_attack(df, process_time):
    """
    A function that filters out the BASH SHELLSHOCK alert data obtained from FireEye
    :param df: a DataFrame object
    :param process_time:
    :return:
    """
    result = []
    track = []
    for o in df:
        if 'METHODOLOGY - WEB APP ATTACK' in o['message']:
            track.append(o)
        else:
            result.append(o)
    trackfile = open('tracking_webAppAttack_' + process_time + '.txt', 'w')
    numskip = 1
    for item in track:
        trackfile.write("\n\n**** {:d}: Display ID {} ****\n\n{}".format(numskip, item['displayId'], item))
        numskip += 1
    return result


def process_alert(file_name):
    """
    A function that removes the alerts property from the FireEye alert and transform the data into a JSON ready format
    :param file_name:
    :return:
    """

    processed_line = open(file_name, 'r').read()
    char_pos = processed_line.find("}")
    new_line = "{" + processed_line[char_pos + 2:]
    return new_line


def main(inputfile):
    ts = TruStar()
    token = ts.get_token()

    df = pd.read_json(process_alert(inputfile))

    process_time = time.strftime('%Y-%m-%d %H:%M', time.localtime(time.time()))
    filtered_falsepositive = filter_false_positive(df, process_time)
    filtered_winmethodology = filter_win_methodology(filtered_falsepositive, process_time)

    filtered_bashshellshock = filter_bash_shellshock(filtered_winmethodology, process_time)

    filtered_data = filter_webapp_attack(filtered_bashshellshock, process_time)

    all_reports = []
    for alert in filtered_data:
        title = str(alert['displayId']) + ' ' + str(alert['message'].encode('utf-8'))
        content = ""
        for key in alert:
            type_value = type(alert[key])
            if type_value == list or type_value == int or type_value == long or type_value == bool \
                    or type_value == dict or alert[key] is None:
                content += key + ': ' + str(alert[key]).replace('u\'', '\'') + '\n'
            else:
                content += key + ': ' + str(alert[key].encode('ascii', 'ignore')) + '\n'
        created_time = str(alert['createDate'])
        current_report = {'reportTitle': title, 'reportContent': content, 'reportDateTime': created_time}
        all_reports.append(current_report)

    if do_enclave_submissions:
        for staged_report in all_reports:
            start_time = time.time()
            response = ts.submit_report(token, staged_report['reportContent'], staged_report['reportTitle'],
                                        discovered_time_str=staged_report['reportDateTime'],
                                        enclave=True)

            print(response)
            if 'error' in response:
                print("Submission failed with error: {}, {}".format(response['error'], response['message']))
                if response['error'] in ("Internal Server Error", "Access token expired", "Authentication error"):
                    print("Auth token expired, requesting new one")
                    token = ts.get_token()
                else:
                    raise Exception
            else:
                end_time = time.time()
                delta_time = end_time - start_time
                print("Submitted report title {} as TruSTAR IR {}".format(staged_report['reportTitle'],
                                                                          response['reportId']) + " Time:" + str(
                    delta_time))
            if 'reportIndicators' in response and len(response['reportIndicators']) > 0:
                print("Extracted the following indicators: {}".format(json.dumps(response['reportIndicators'])))

            print()
            time.sleep(3)


if __name__ == '__main__':
    main(sys.argv[1])



# git tag 0.0.12 -m "Bump version to 0.0.12" && git push --tags
# python setup.py register -r pypitest
# python setup.py sdist upload -r pypitest
# python setup.py register -r pypi
# python setup.py sdist upload -r pypi

from setuptools import setup, find_packages

setup(
    name='trustar',
    # packages= ['trustar'],
    version='0.0.12',
    author='TruSTAR Technology Inc.',
    author_email='support@trustar.co',
    url='https://github.com/trustar/trustar-python',
    download_url='https://github.com/trustar/trustar-python/tarball/0.0.12',
    description='Python SDK for the TruSTAR REST API',
    license='MIT',
    install_requires=['future',
                      'python-dateutil',
                      'pytz',
                      'requests',
                      'configparser', 'unicodecsv', 'pdfminer'],
    # package source directory
    # package_dir={'': 'trustar'},
    packages=find_packages(),
    use_2to3=True
)



from __future__ import print_function
import unicodecsv
import csv
import argparse
import json

from trustar import TruStar

FILE_NAME = 'latest-indicators.csv'

# Default string used to query all indicator types extracted by TruSTAR
DEFAULT_TYPE_STRING = 'ALL'

# Default number of indicators returned
DEFAULT_LIMIT = 5000

# Default interval size
DEFAULT_INTERVAL_SIZE = 24

# All the indicators types extracted by TruSTAR
ALL_TYPES = ['REGISTRY_KEY',
             'SHA1',
             'MD5',
             'URL',
             'IP',
             'EMAIL_ADDRESS',
             'BITCOIN_ADDRESS',
             'MALWARE',
             'SHA256',
             'CVE',
             'SOFTWARE']


def save_to_file(all_data, file_name, source, indicator_types):
    """
    Lists of indicators grouped by type
    :param all_data:
    :param file_name:
    :param source:
    :param indicator_types:
    :return:
    """
    if indicator_types == DEFAULT_TYPE_STRING:
        indicator_types = ALL_TYPES

    with open(file_name, 'wd') as csvfile:
        writer = unicodecsv.writer(csvfile, delimiter=",", quotechar="\"", quoting=csv.QUOTE_MINIMAL)
        for indicator_type in indicator_types:
            if len(all_data[indicator_type]) != 0:
                for data in all_data[indicator_type]:
                    current = data + "," + indicator_type + "," + source
                    writer.writerow(current.split(","))


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=('Query TruSTAR indicators and store them in a CSV file\n'
                                                  'Example:\n\n'
                                                  'python latest_indicators.py -s' +
                                                  ' INCIDENT_REPORT -t "IP,URL" -l 50 -i 12'))
    parser.add_argument('-s', '--source', required=True, dest='source', help='Source can be INCIDENT_REPORT or OSINT')
    parser.add_argument('-t', '--types', required=False, dest='types', help='Types of indicators')
    parser.add_argument('-l', '--limit', required=False, dest='limit',
                        help='Limit on the returned number of indicators')
    parser.add_argument('-i', '--intervalSize', required=False, dest='interval_size',
                        help='Interval size in hours')
    parser.add_argument('-f', '--fileName', required=False, dest='file_name')

    ts = TruStar(config_role="trustar")

    args = parser.parse_args()

    source_type = args.source
    if args.types:
        indicator_types = args.types.split(",")
    else:
        indicator_types = DEFAULT_TYPE_STRING

    if args.limit:
        limit = args.limit
    else:
        limit = DEFAULT_LIMIT

    if args.interval_size:
        interval_size = args.interval_size
    else:
        interval_size = DEFAULT_INTERVAL_SIZE

    if args.file_name:
        file_name = args.file_name
    else:
        file_name = FILE_NAME

    response = ts.query_latest_indicators(ts.get_token(), source_type, indicator_types, limit, interval_size)
    print(json.dumps(response, indent=2))
    save_to_file(response['indicators'], file_name, source_type, indicator_types)


if __name__ == '__main__':
    main()
